searchData={"items":[{"type":"task","title":"mix systemd.generate","doc":"Create systemd unit files for project.","ref":"Mix.Tasks.Systemd.Generate.html"},{"type":"task","title":"Usage - mix systemd.generate","doc":"# Create unit files\n    MIX_ENV=prod mix systemd.generate","ref":"Mix.Tasks.Systemd.Generate.html#module-usage"},{"type":"task","title":"mix systemd.init","doc":"Initialize template files.","ref":"Mix.Tasks.Systemd.Init.html"},{"type":"task","title":"Command line options - mix systemd.init","doc":"* `--template_dir` - target directory","ref":"Mix.Tasks.Systemd.Init.html#module-command-line-options"},{"type":"task","title":"Usage - mix systemd.init","doc":"# Copy default templates into your project\n    mix systemd.init","ref":"Mix.Tasks.Systemd.Init.html#module-usage"},{"type":"extras","title":"README","doc":"![test workflow](https://github.com/cogini/mix_systemd/actions/workflows/test.yml/badge.svg)\n[![Module Version](https://img.shields.io/hexpm/v/mix_systemd.svg)](https://hex.pm/packages/mix_systemd)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/mix_systemd/)\n[![Total Download](https://img.shields.io/hexpm/dt/mix_systemd.svg)](https://hex.pm/packages/mix_systemd)\n[![License](https://img.shields.io/hexpm/l/mix_systemd.svg)](https://github.com/cogini/mix_systemd/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/cogini/mix_systemd.svg)](https://github.com/cogini/mix_systemd/commits/master)\n[![Contributor Covenant](https://img.shields.io/badge/Contributor%20Covenant-2.1-4baaaa.svg)](CODE_OF_CONDUCT.md)\n\n# mix_systemd\n\nThis library generates a\n[systemd](https://www.freedesktop.org/software/systemd/man/systemd.unit.html)\nunit file to manage an Elixir release. It supports releases generated by Elixir 1.9+\n[mix release](https://hexdocs.pm/mix/Mix.Tasks.Release.html) or\n[Distillery](https://hexdocs.pm/distillery/home.html).\n\nAt its heart, it's a mix task which reads information about the project from\n`mix.exs` and `config/config.exs` then generates systemd unit files using Eex\ntemplates. The goal is that the project defaults will generate a good systemd\nunit file, and standard options support more specialized use cases.\n\nIt uses standard systemd functions and conventions to make your app a more\n\"native\" OS citizen and takes advantage of systemd features to improve\nsecurity and reliability. While it can be used standalone, more advanced use\ncases use scripts from e.g., [mix_deploy](https://github.com/cogini/mix_deploy).\n\nThis [complete example app](https://github.com/cogini/mix-deploy-example) puts the\npieces together.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `mix_systemd` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mix_systemd, \"~> 0.7\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - README","doc":"The library tries to choose reasonable defaults, so you may not need to\nconfigure anything. It reads the app name from `mix.exs` and calculates default\nvalues for its configuration parameters. For example, if your app is named\n`foo_bar`, it will create a service named `foo-bar`, deployed to\n`/srv/foo-bar`, running under the user `foo-bar`.\n\nYou can override these parameters using settings in `config/config.exs`, e.g.:\n\n```elixir\nconfig :mix_systemd,\n  app_user: \"app\",\n  app_group: \"app\",\n  base_dir: \"/opt\",\n  env_vars: [\n    \"PORT=8080\",\n  ]\n```","ref":"readme.html#configuration"},{"type":"extras","title":"Configuration strategies - README","doc":"There are four different kinds of things that we may want to configure:\n\n1. Static information about application layout, e.g., file paths.\n   This is the same for all machines in an environment, e.g., staging or prod.\n\n2. Information specific to the environment, e.g., the hostname of the db\n   server.\n\n3. Secrets such as db passwords, API keys, or encryption keys.\n\n4. Dynamic information such as the IP address of the server or other\n   machines in the cluster.\n\nElixir has a couple of mechanisms for storing configuration. When you compile\nthe release, it converts Elixir-format config files like `config/config.exs`\ninto an initial application environment that is read by `Application.get_env/3`.\nThat's good for simple, relatively static apps. It's not ideal to store\npasswords in the release file, though.\n\nElixir 1.9+ releases support dynamic configuration at runtime. You can configure\nvia the Elixir file `config/runtime.exs` which is loaded when the VM boots or\nuse the shell script `rel/env.sh.eex` to set environment vars.\n\nWith these you can theoretically do anything. In practice, however, it can be\nmore convenient and secure to process the config outside of the app. That's\nwhere `mix_systemd` and `mix_deploy` come in.","ref":"readme.html#configuration-strategies"},{"type":"extras","title":"Environment vars - README","doc":"The simplest thing is to set environment variables. Add individual vars to\n`env_vars`, and they will be set in the systemd unit file. Add files to\n`env_files` and systemd will load them before starting your app.\nYour application then calls `System.get_env/1` in `config/runtime.exs` or\napplication startup. Note that these environment vars are read at *runtime*,\nnot when building your app.\n\n```elixir\nenv_vars: [\n  # Set a variable, good for things that are not sensitive and don't change\n  \"PORT=8080\",\n],\ndirs: [\n  # create /etc/foo\n  :configuration,\n],\nenv_files: [\n  # Read environment vars from the file /etc/foo/environment\n  [\"-\", :configuration_dir, \"/environment\"],\n]\n```\n\n`/etc/foo/environment` looks like:\n\n    DATABASE_URL=\"ecto://foo_prod:Sekrit!@db.foo.local/foo_prod\"\n    SECRET_KEY_BASE=\"EOdJB1T39E5Cdeebyc8naNrOO4HBoyfdzkDy2I8Cxiq4mLvIQ/0tK12AK1ahrV4y\"\n    HOST=\"www.example.com\"\n    ASSETS_HOST=\"assets.example.com\"\n    RELEASE_COOKIE=\"LmCMGNz04yEJ4MQc6jt3cS7QjAppYOw_bQa7NE5hPZJGqL3Yry1jUg==\"\n\n`config/runtime.exs` does something like the following (the default files are good):\n\n```elixir\nconfig :foo, Foo.Repo,\n  url: System.get_env(\"DATABASE_URL\")\n\nconfig :foo, FooWeb.Endpoint,\n  http: [:inet6, port: System.get_env(\"PORT\") || 4000],\n  url: [host: System.get_env(\"HOST\"), port: 443],\n  static_url: [host: System.get_env(\"ASSETS_HOST\"), port: 443],\n  secret_key_base: System.get_env(\"SECRET_KEY_BASE\"),\n  cache_static_manifest: \"priv/static/cache_manifest.json\"\n```","ref":"readme.html#environment-vars"},{"type":"extras","title":"Copying files - README","doc":"The question is how to get the environment files onto the server. For simple\nserver deployments, we can copy the config to the server when doing the initial\nsetup.\n\nIn cloud environments, we may run from a read-only image, e.g., an Amazon AMI,\nwhich gets configured at start up based on the environment by copying the\nconfig from an S3 bucket, e.g.:\n\n```shell\numask 077\naws s3 sync --exact-timestamps --no-progress \"s3://${CONFIG_BUCKET}/\" \"/etc/foo/\"\nchown -R $DEPLOY_USER:$APP_GROUP /etc/foo\nfind /etc/foo -type f -exec chmod 640 {} \\;\nfind /etc/foo -type d -exec chmod 750 {} \\;\n```\n\nThe following example runs the script `/srv/foo/bin/deploy-sync-config-s3` from\n`mix_deploy`.  It uses an environment file in `/srv/foo/etc/environment`\nto bootstrap the sync, e.g., setting the S3 bucket name. That file\nis placed there by CodeDeploy at deploy time.\n\n```elixir\nconfig :mix_systemd,\n  exec_start_pre: [\n    # Run before starting the app\n    # The `!` means the script is run as root, not as the app user\n    [\"!\", :deploy_dir, \"/bin/deploy-sync-config-s3\"]\n  ],\n  dirs: [\n    :configuration, # /etc/foo, app configuration, e.g. db passwords\n    :runtime,       # /run/foo, temp files which may be deleted between runs\n  ],\n  env_files: [\n    [\"-\", :deploy_dir, \"/etc/environment\"], # /srv/foo/etc/environment\n  ]\n  env_vars: [\n    # Tell release to use /run/foo for temp files\n    [\"RELEASE_TMP=\", :runtime_dir],\n  ]\n```","ref":"readme.html#copying-files"},{"type":"extras","title":"Config providers - README","doc":"At a certain point, making everything into an environment var becomes annoying.\nIt's verbose and vars are simple strings, so you have to encode values\nsafely and convert them back to lists, integers or atoms.\n\n[Config providers](https://hexdocs.pm/elixir/Config.Provider.html) let you load\nfiles in standard formats like [TOML](https://hexdocs.pm/toml_config/readme.html).\n\n```toml\n[foo.\"Foo.Repo\"]\nurl = \"ecto://foo_prod:Sekrit!@db.foo.local/foo_prod\"\npool_size = 15\n\n[foo.\"FooWeb.Endpoint\"]\nsecret_key_base = \"EOdJB1T39E5Cdeebyc8naNrOO4HBoyfdzkDy2I8Cxiq4mLvIQ/0tK12AK1ahrV4y\"\n```\n\nThe app reads these config files on startup and merges them into the app\nconfig.\n\n```elixir\ndefp releases do\n   [\n     prod: [\n       include_executables_for: [:unix],\n       config_providers: [\n         {TomlConfigProvider, path: \"/etc/foo/config.toml\"}\n       ],\n       steps: [:assemble, :tar]\n     ]\n   ]\n end\n```\n\nThe startup scripts read the initial application environment compiled into the\nrelease, parse the config file, merge the values, write it to a temp file, then\nstart the VM. Because of that, they need a writable directory. That is\nconfigured using the `RELEASE_TMP` environment var, normally set to the app's\n`runtime_dir`.\n\n```elixir\ndirs: [\n  :configuration,\n  :runtime,\n],\nenv_vars: [\n  [\"RELEASE_TMP=\", :runtime_dir],\n],\n```","ref":"readme.html#config-providers"},{"type":"extras","title":"Config servers and vaults - README","doc":"You can also store config params in an external configuration system and\nread them at runtime. An example is [AWS Systems Manager Parameter\nStore](https://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-paramstore.html).\n\nSet a parameter using the AWS CLI:\n\n```shell\naws ssm put-parameter --name '/foo/prod/db/password' --type ‘SecureString’ --value 'Sekrit!\"\n```\n\nWhile it's possible to read params in `config/runtime.exs`, it's tedious.\nBetter is to grab all of them at once and write them to a file, then read it in\nwith a Config Provider like [aws_ssm_provider](https://github.com/caredox/aws_ssm_provider).\n\n```shell\naws --region us-east-1 ssm get-parameters-by-path --path \"/foo/prod/\" --recursive --with-decryption --query \"Parameters[]\" > /etc/foo/ssm.json\n```\n\n```elixir\ndefp releases do\n   [\n     prod: [\n       include_executables_for: [:unix],\n       config_providers: [\n         {AwsSsmProvider, path: \"/etc/foo/ssm.json\"}\n       ],\n       steps: [:assemble, :tar]\n     ]\n   ]\n end\n```","ref":"readme.html#config-servers-and-vaults"},{"type":"extras","title":"Dynamic config - README","doc":"You can write code to do things like query the system for the primary IP\naddress, but `cloud-init`\n[already does it](https://cloudinit.readthedocs.io/en/latest/topics/instancedata.html).\nYou just have to read the JSON file.\n\nThe most common use for this is setting up the VM node name. In `env.sh`:\n\n```shell\nCLOUD_NAME=$(jq -r '.v1.cloud_name' < /run/cloud-init/instance-data.json)\nif [ \"$CLOUD_NAME\" = \"digitalocean\" ]; then\n    IP_ADDR=$(jq -r '.ds.meta_data.interfaces.public[0].anchor_ipv4.ip_address' < /run/cloud-init/instance-data.json)\n    DEFAULT_IPV4=\"$IP_ADDR\"\nelif [ \"$CLOUD_NAME\" = \"aws\" ]; then\n    IP_ADDR=$(jq -r '.ds.meta_data.\"local-ipv4\"' < /run/cloud-init/instance-data.json)\n    # IP_ADDR=$(jq -r '.ds.meta_data.\"public-ipv4\"' < /run/cloud-init/instance-data.json)\n    AWS_REGION=$(jq -r '.v1.region' < /run/cloud-init/instance-data.json)\nfi\nRELEASE_DISTRIBUTION=\"name\"\nRELEASE_NODE=\"${RELEASE_NAME}@${IP_ADDR}\"\n```","ref":"readme.html#dynamic-config"},{"type":"extras","title":"Security - README","doc":"An important security principle is\n\"[least privilege](https://www.cogini.com/blog/improving-app-security-with-the-principle-of-least-privilege/)\".\nIf an attacker manages to compromise the app, then they can do whatever it has\npermissions to do, not just what you expect. Because of that, I prefer that the\naccount that the app runs under cannot write files, and having a writable\nconfig file that is also executed is the worst case scenario.","ref":"readme.html#security"},{"type":"extras","title":"Usage - README","doc":"First, use the `systemd.init` task to template files from the library to the\n`rel/templates/systemd` directory in your project.\n\n```shell\nmix systemd.init\n```\n\nNext, generate output files in the build directory under\n`_build/#{mix_env}/systemd/lib/systemd/system`.\n\n```shell\nMIX_ENV=prod mix systemd.generate\n```","ref":"readme.html#usage"},{"type":"extras","title":"Configuration options - README","doc":"The following sections describe common configuration options.\nSee `lib/mix/tasks/systemd.ex` for the details of more obscure options.\n\nIf you need to make changes not supported by the config options,\nthen you can check the templates in `rel/templates/systemd`\ninto source control and make your own changes.  Contributions are welcome!","ref":"readme.html#configuration-options"},{"type":"extras","title":"Basics - README","doc":"`app_name`: Elixir application name, an atom, from the `app` or `app_name`\nfield in the `mix.exs` project. For umbrella apps, set `app_name`.\n\n`module_name`: Elixir camel case module name version of `app_name`, e.g.,\n`FooBar`.\n\n`release_name`: Name of release, default `app_name`.\n\n`ext_name`: External name, used for files and directories, default `app_name`\nwith underscores converted to \"-\", e.g., `foo-bar`.\n\n`service_name`: Name of the systemd service, default `ext_name`.\n\n`release_system`: `:mix | :distillery`, default `:mix`\n\nIdentifies the system used to generate the releases,\n[Mix](https://hexdocs.pm/mix/Mix.Tasks.Release.html) or\n[Distillery](https://hexdocs.pm/distillery/home.html).","ref":"readme.html#basics"},{"type":"extras","title":"Users - README","doc":"`app_user`: OS user account that the app runs under, default `ext_name`.\n\n`app_group`: OS group account, default `ext_name`.","ref":"readme.html#users"},{"type":"extras","title":"Directories - README","doc":"`base_dir`: Base directory for app files on target, default `/srv`.\n\n`deploy_dir`: Directory for app files on target, default `#{base_dir}/#{ext_name}`.\n\nWe use the\n[standard app directories](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#RuntimeDirectory=),\nfor modern Linux systems. App files are under `/srv`, configuration under\n`/etc`, transient files under `/run`, data under `/var/lib`.\n\nDirectories are named based on the app name, e.g. `/etc/#{ext_name}`.\nThe `dirs` variable specifies which directories the app uses.\nBy default, it doesn't set up anything. To enable them, configure the `dirs`\nparam, e.g.:\n\n```elixir\ndirs: [\n  # :runtime,       # App runtime files which may be deleted between runs, /run/#{ext_name}\n  # :configuration, # App configuration, e.g. db passwords, /etc/#{ext_name}\n  # :state,         # App data or state persisted between runs, /var/lib/#{ext_name}\n  # :cache,         # App cache files which can be deleted, /var/cache/#{ext_name}\n  # :logs,          # App external log files, not via journald, /var/log/#{ext_name}\n  # :tmp,           # App temp files, /var/tmp/#{ext_name}\n],\n```\n\nRecent versions of systemd (since 235) will create these directories at\nstart time based on the settings in the unit file. With earlier systemd\nversions, create them beforehand using installation scripts, e.g.,\n[mix_deploy](https://github.com/cogini/mix_deploy).\n\nFor security, we set permissions to 750, more restrictive than the systemd\ndefaults of 755. You can configure them with variables like\n`configuration_directory_mode`. See the defaults in\n`lib/mix/tasks/systemd.ex`.\n\n`systemd_version`: Sets the systemd version on the target system, default 235.\nThis determines which systemd features the library will enable. If you are\ntargeting an older OS release, you may need to change it. Here are the systemd\nversions in common OS releases:\n\n* CentOS 7: 219\n* Ubuntu 16.04: 229\n* Ubuntu 18.04: 237","ref":"readme.html#directories"},{"type":"extras","title":"Additional directories - README","doc":"The library uses a directory structure under `deploy_dir` which supports\nmultiple releases, similar to [Capistrano](https://capistranorb.com/documentation/getting-started/structure/).\n\n* `scripts_dir`: deployment scripts which, e.g., start and stop the unit, default `bin`.\n* `current_dir`: where the current Erlang release is unpacked or referenced by symlink, default `current`.\n* `releases_dir`: where versioned releases are unpacked, default `releases`.\n* `flags_dir`: dir for flag files to trigger restart, e.g., when `restart_method` is `:systemd_flag`, default `flags`.\n\nWhen using multiple releases and symlinks, the deployment process works as follows:\n\n1. Create a new directory for the release with a timestamp like\n   `/srv/foo/releases/20181114T072116`.\n\n2. Upload the new release tarball to the server and unpack it to the releases dir.\n\n3. Make a symlink from `/srv/#{ext_name}/current` to the new release dir.\n\n4. Restart the app.\n\nIf you are only keeping a single version, then deploy it to the directory\n`/srv/#{ext_name}/current`.","ref":"readme.html#additional-directories"},{"type":"extras","title":"Variable expansion - README","doc":"The following variables support variable expansion:\n\n```elixir\nexpand_keys: [\n  :env_files,\n  :env_vars,\n  :runtime_environment_service_script,\n  :exec_start_pre,\n  :exec_start_wrap,\n  :read_write_paths,\n  :read_only_paths,\n  :inaccessible_paths,\n]\n```\n\nYou can specify values as a list of terms, and it will look up atoms as keys in\nthe config. This lets you reference, e.g., the deploy dir or configuration dir without\nhaving to specify the full path, e.g., `[\"!\", :deploy_dir, \"/bin/myscript\"]` gets\nconverted to `\"!/srv/foo/bin/myscript\"`.","ref":"readme.html#variable-expansion"},{"type":"extras","title":"Environment vars - README","doc":"The library sets env vars in the unit file:\n\n* `MIX_ENV`: `mix_env`, default `Mix.env()`\n* `LANG`: `env_lang`, default `en_US.utf8`\n\n* `RUNTIME_DIR`: `runtime_dir`, if `:runtime` in `dirs`\n* `CONFIGURATION_DIR`: `configuration_dir`, if `:configuration` in `dirs`\n* `LOGS_DIR`: `logs_dir`, if `:logs` in `dirs`\n* `CACHE_DIR`: `cache_dir`, if `:cache` in `dirs`\n* `STATE_DIR`: `state_dir`, if `:state` in `dirs`\n* `TMP_DIR`: `tmp_dir`, if `:tmp` in `dirs`\n\nYou can set additional vars using `env_vars`, e.g.:\n\n```elixir\nenv_vars: [\n  \"PORT=8080\",\n]\n```\nYou can also reference the value of other parameters by name, e.g.:\n\n```elixir\nenv_vars: [\n  [\"RELEASE_TMP=\", :runtime_dir],\n]\n```\n\nYou can read environment vars from files with `env_files`, e.g.:\n\n```elixir\nenv_files: [\n  [\"-\", :deploy_dir, \"/etc/environment\"],\n  [\"-\", :configuration_dir, \"environment\"],\n  [\"-\", :runtime_dir, \"environment\"],\n],\n```\n\nThe \"-\" at the beginning makes the file optional; the system will start without it.\nLater values override earlier values, so you can set defaults which get\noverridden in the local or runtime environment.","ref":"readme.html#environment-vars"},{"type":"extras","title":"Runtime dirs - README","doc":"The release scripts may need to write temp files and log files, e.g., when\ngenerating the application config files. By default, they do this under\nthe release dir, e.g., `/srv/foo/current/tmp`.\n\nFor security, it's better to deploy the app using a different user account from\nthe one that the app runs under, with the source files read only. This makes\nit harder for an attacker to make changes to the source and then have the app\nrun them.\n\nIn that case, we need to set an environment var which tells the release\nstartup scripts where they can write files. For Mix releases, that is\n`RELEASE_TMP` and for Distillery it is `RELEASE_MUTABLE_DIR`, e.g.:\n\n```elixir\nenv_vars: [\n  {\"RELEASE_TMP=\", :runtime_dir},\n]\n```\n\nBy default systemd will delete the runtime directory when restarting the app.\nThis can be annoying when debugging startup issues. You can set\n`runtime_directory_preserve` to `restart` or `yes` (see\n[RuntimeDirectoryPreserve](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#RuntimeDirectoryPreserve=)).","ref":"readme.html#runtime-dirs"},{"type":"extras","title":"Starting and restarting - README","doc":"The following variables set systemd variables:\n\n`service_type`: `:simple | :exec | :notify | :forking`. systemd\n[Type](https://www.freedesktop.org/software/systemd/man/systemd.service.html#Type=), default `:simple`.\n\nModern applications don't fork, they run in the foreground and rely on the\nsupervisor to manage them as a daemon. This is done by setting `service_type`\nto `:simple` or `:exec`. Note that in `simple` mode, systemd doesn't actually\ncheck if the app started successfully, it just continues starting other units.\nIf something depends on your app being up, `:exec` may be better.\n\nSet `service_type` to `:forking`, and the library sets `pid_file` to\n`#{runtime_directory}/#{app_name}.pid` and sets the `PIDFILE` env var to tell\nthe boot scripts where it is.\n\nThe Erlang VM runs pretty well in foreground mode, but traditionally runs as\nas a standard Unix-style daemon, so forking might be better. Systemd\nexpects foregrounded apps to die when their pipe closes. See\nhttps://elixirforum.com/t/systemd-cant-shutdown-my-foreground-app-cleanly/14581/2\n\n`restart_method`: `:systemctl | :systemd_flag | :touch`, default `:systemctl`\n\nSet this to `:systemd_flag`, and the library will generate an additional\nunit file which watches for changes to a flag file and restarts the\nmain unit. This allows updates to be pushed to the target machine by an\nunprivileged user account which does not have permissions to restart\nprocesses. Touch the file `#{flags_dir}/restart.flag` and systemd will\nrestart the unit.\n\n`working_dir`: Current working dir for app. systemd\n[WorkingDirectory](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#WorkingDirectory=),\ndefault `current_dir`.\n\n`limit_nofile`: Limit on open files, systemd\n[LimitNOFILE](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#LimitCPU=),\ndefault 65535.\n\n`umask`: Process umask, systemd\n[UMask](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#UMask=),\ndefault \"0027\". Note that this is octal, so it needs to be a string.\n\n`restart_sec`: Time in seconds to wait between restarts, systemd\n[RestartSec](https://www.freedesktop.org/software/systemd/man/systemd.service.html#RestartSec=),\ndefault 100ms.\n\n`syslog_identifier`: Logging name, systemd\n[SyslogIdentifier](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#SyslogIdentifier=),\ndefault `service_name`\n\n\n## `ExecStartPre` scripts\n\nScripts specified in `exec_start_pre` (systemd\n[ExecStartPre](https://www.freedesktop.org/software/systemd/man/systemd.service.html#ExecStartPre=)])\nrun before the main `ExecStart` script runs, e.g.:\n\n```elixir\nexec_start_pre: [\n  [\"!\", :deploy_dir, \"/bin/deploy-sync-config-s3\"]\n]\n```\n\nThis runs the `deploy-sync-config-s3` script from `mix_deploy`, which\ncopies config files from an S3 bucket into `/etc/foo`. By default,\nscripts run as the same user and group as the main script. Putting\n`!` in front makes the script run with [elevated\nprivileges](https://www.freedesktop.org/software/systemd/man/systemd.service.html#ExecStart=),\nallowing it to write config to `/etc/foo` even if the main user account cannot for security reasons.\n\n#### ExecStart wrapper script\n\nInstead of running the main `ExecStart` script directly, you can run a shell script\nwhich sets up the environment, then runs the main script with `exec`.\nSet `exec_start_wrap` to the name of the script, e.g.\n`deploy-runtime-environment-wrap` from `mix_deploy`.\n\nIn Elixir 1.9+ releases you can use `env.sh`, but this runs earlier\nwith elevated permissions, so a wrapper script may still be useful.\n\n#### Runtime environment service\n\nYou can run your own separate service to configure the runtime environment\nbefore the app runs.  Set `runtime_environment_service_script` to a script such\nas `deploy-runtime-environment-file` from `mix_deploy`. This library will\ncreate a `#{service_name}-runtime-environment.service` unit and make it a\nsystemd runtime dependency of the app.","ref":"readme.html#starting-and-restarting"},{"type":"extras","title":"Runtime dependencies - README","doc":"Systemd starts units in parallel when possible. To enforce ordering, set\n`unit_after_targets` to the names of systemd units that this unit depends on.\nFor example, if this unit should run after cloud-init to get [runtime network\ninformation](https://cloudinit.readthedocs.io/en/latest/topics/network-config.html#network-configuration-outputs),\nset:\n\n```elixir\nunit_after_targets: [\n  \"cloud-init.target\"\n]\n```","ref":"readme.html#runtime-dependencies"},{"type":"extras","title":"Security - README","doc":"`paranoia`: Enable systemd security options, default `false`.\n\n    NoNewPrivileges=yes\n    PrivateDevices=yes\n    PrivateTmp=yes\n    ProtectSystem=full\n    ProtectHome=yes\n    PrivateUsers=yes\n    ProtectKernelModules=yes\n    ProtectKernelTunables=yes\n    ProtectControlGroups=yes\n    MountAPIVFS=yes\n                                                                                                    │\n`chroot`: Enable systemd [chroot](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#RootDirectory=), default `false`.\nSets systemd `RootDirectory` is set to `current_dir`. You can also set systemd [ReadWritePaths=, ReadOnlyPaths=,\nInaccessiblePaths=](https://www.freedesktop.org/software/systemd/man/systemd.exec.html#ReadWritePaths=)\nwith the `read_write_paths`, `read_only_paths` and `inaccessible_paths` vars, respectively.","ref":"readme.html#security"},{"type":"extras","title":"Distillery - README","doc":"Distillery has largely been replaced by Elixir native releases.\nThis library works fine with it, though. `exec_start_pre` scripts\nare particularly useful in the absence of `env.sh`.\n\nConfigure the library by setting `release_system: :distillery`, e.g..\n\n```elixir\nconfig :mix_systemd,\n  release_system: :distillery,\n  exec_start_pre: [\n    # Run script as root before starting\n    [\"!\", :deploy_dir, \"/bin/deploy-sync-config-s3\"]\n  ],\n  dirs: [\n    :configuration,\n    :runtime,\n  ],\n  runtime_directory_preserve: \"yes\",\n  env_vars: [\n    # Use /run/foo for temp files\n    [\"RELEASE_MUTABLE_DIR=\", :runtime_dir],\n    # expand $CONFIGURATION_DIR in config files\n    REPLACE_OS_VARS=true,\n  ]\n```\n\nSet up\n[config providers](https://hexdocs.pm/distillery/Mix.Releases.Config.Providers.Elixir.html)\nin `rel/config.exs`:\n\n```elixir\nenvironment :prod do\n  set config_providers: [\n    {Mix.Releases.Config.Providers.Elixir, [\"${CONFIGURATION_DIR}/config.exs\"]}\n  ]\nend\n```\n\nThis reads files in Elixir config format. Instead of including your\n`prod.secret.exs` file in `prod.exs`, you can copy it to the server separately,\nand it will be read at startup.\n\nThe [TOML configuration provider](https://github.com/bitwalker/toml-elixir) works similarly:\n\n```elixir\nenvironment :prod do\n  set config_providers: [\n    {Toml.Provider, [path: \"${CONFIGURATION_DIR}/config.toml\"]},\n  ]\nend\n```\n\nAdd the TOML config provider to `mix.exs`:\n\n```elixir\n{:toml_config_provider, \"~> 0.2.0\"}\n```\n\nYou can generate a file under the release with an overlay in\n`rel/config.exs`, e.g.:\n\n```elixir\nenvironment :prod do\n  set overlays: [\n    {:mkdir, \"etc\"},\n    {:copy, \"rel/etc/environment\", \"etc/environment\"},\n    # {:template, \"rel/etc/environment\", \"etc/environment\"}\n  ]\nend\n```\n\nThat results in a file that would be read by:\n\n```elixir\nenv_files: [\n  [\"-\", :current_dir, \"/etc/environment\"],\n],\n```\n\nDocumentation is here: https://hexdocs.pm/mix_systemd\n\nThis project uses the Contributor Covenant version 2.1. Check [CODE_OF_CONDUCT.md](/CODE_OF_CONDUCT.md) for more information.\n\n# Contacts\n\nI am `jakemorrison` on on the Elixir Slack and Discord, `reachfh` on Freenode\n`#elixir-lang` IRC channel. Happy to chat or help with your projects.","ref":"readme.html#distillery"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.8.1] - 2024-07-07","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- Update libs\n\n## [0.8.0] - 2024-05-29","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Allow setting app name in config and set sane default for umbrella apps,\n  thanks to @probably-not\n- Update library versions\n- Add static checks\n- Add GitHub Actions CI\n\n## [0.7.5] - 2020-10-07","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Update dialyxir version\n\n## [0.7.4] - 2020-10-07","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Make output_dir overridable from config, thanks to @yuchunc\n- Update dependencies for Elixir 1.11\n\n## [0.7.3] - 2020-02-25","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Update docs\n\n## [0.7.1] - 2020-02-12","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Bring back `env_lang` variable to set `LANG` environment var\n- Default LANG to `en_US.utf8` for better compatibility\n- Update ex_doc 0.21.2 => 0.21.3 and build all the time\n\n## [0.7.0] - 2020-01-01","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Support Elixir 1.9 `mix release`\n- Support variable references in paths","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed obsolete option flags","ref":"changelog.html#removed"},{"type":"extras","title":"License (Apache-2.0)","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - License (Apache-2.0)","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - License (Apache-2.0)","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - License (Apache-2.0)","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - License (Apache-2.0)","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - License (Apache-2.0)","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - License (Apache-2.0)","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - License (Apache-2.0)","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - License (Apache-2.0)","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2021-2023, Jake Morrison  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"},{"type":"extras","title":"Contributing","doc":"# Contributing to Uinta\n\nPlease take a moment to review this document in order to make the contribution\nprocess easy and effective for everyone involved!\nAlso make sure you read our [Code of Conduct](CODE_OF_CONDUCT.md) that outlines our commitment towards an open and welcoming environment.","ref":"contributing.html"},{"type":"extras","title":"Using the issue tracker - Contributing","doc":"Use the issues tracker for:\n\n* [Bug reports](#bug-reports)\n* [Submitting pull requests](#pull-requests)\n\nWe do our best to keep the issue tracker tidy and organized, making it useful\nfor everyone. For example, we classify open issues per perceived difficulty,\nmaking it easier for developers to [contribute to Uinta](#pull-requests).","ref":"contributing.html#using-the-issue-tracker"},{"type":"extras","title":"Bug reports - Contributing","doc":"A bug is either a _demonstrable problem_ that is caused by the code in the repository,\nor indicate missing, unclear, or misleading documentation. Good bug reports are extremely\nhelpful - thank you!\n\nGuidelines for bug reports:\n\n1. **Use the GitHub issue search** &mdash; check if the issue has already been\n   reported.\n\n2. **Check if the issue has been fixed** &mdash; try to reproduce it using the\n   `master` branch in the repository.\n\n3. **Isolate and report the problem** &mdash; ideally create a reduced test\n   case.\n\nPlease try to be as detailed as possible in your report. Include information about\nyour Operating System, as well as your Erlang, Elixir and Uinta versions. Please provide steps to\nreproduce the issue as well as the outcome you were expecting! All these details\nwill help developers to fix any potential bugs.\n\nExample:\n\n> Short and descriptive example bug report title\n>\n> A summary of the issue and the environment in which it occurs. If suitable,\n> include the steps required to reproduce the bug.\n>\n> 1. This is the first step\n> 2. This is the second step\n> 3. Further steps, etc.\n>\n> ` ` - a link to the reduced test case (e.g. a GitHub Gist)\n>\n> Any other information you want to share that is relevant to the issue being\n> reported. This might include the lines of code that you have identified as\n> causing the bug, and potential solutions (and your opinions on their\n> merits).","ref":"contributing.html#bug-reports"},{"type":"extras","title":"Contributing Documentation - Contributing","doc":"Code documentation (`@doc`, `@moduledoc`, `@typedoc`) has a special convention:\nthe first paragraph is considered to be a short summary.\n\nFor functions, macros and callbacks say what it will do. For example write\nsomething like:\n\n```elixir\n@doc \"\"\"\nMarks the given value as HTML safe.\n\"\"\"\ndef safe({:safe, value}), do: {:safe, value}\n```\n\nFor modules, protocols and types say what it is. For example write\nsomething like:\n\n```elixir\ndefmodule MyModule do\n  @moduledoc \"\"\"\n  Conveniences for working HTML strings and templates.\n  ...\n  \"\"\"\n```\n\nKeep in mind that the first paragraph might show up in a summary somewhere, long\ntexts in the first paragraph create very ugly summaries. As a rule of thumb\nanything longer than 80 characters is too long.\n\nTry to keep unnecessary details out of the first paragraph, it's only there to\ngive a user a quick idea of what the documented \"thing\" does/is. The rest of the\ndocumentation string can contain the details, for example when a value and when\n`nil` is returned.\n\nIf possible include examples, preferably in a form that works with doctests.\nThis makes it easy to test the examples so that they don't go stale and examples\nare often a great help in explaining what a function does.","ref":"contributing.html#contributing-documentation"},{"type":"extras","title":"Pull requests - Contributing","doc":"Good pull requests - patches, improvements, new features - are a fantastic\nhelp. They should remain focused in scope and avoid containing unrelated\ncommits.\n\n**IMPORTANT**: By submitting a patch, you agree that your work will be\nlicensed under the license used by the project.\n\nIf you have any large pull request in mind (e.g. implementing features,\nrefactoring code, etc), **please ask first** otherwise you risk spending\na lot of time working on something that the project's developers might\nnot want to merge into the project.\n\nPlease adhere to the coding conventions in the project (indentation,\naccurate comments, etc.) and don't forget to add your own tests and\ndocumentation. When working with git, we recommend the following process\nin order to craft an excellent pull request:\n\n1. [Fork](https://help.github.com/articles/fork-a-repo/) the project, clone your fork,\n   and configure the remotes:\n\n   ```bash\n   # Clone your fork of the repo into the current directory\n   git clone https://github.com/ /uinta\n\n   # Navigate to the newly cloned directory\n   cd uinta\n\n   # Assign the original repo to a remote called \"upstream\"\n   git remote add upstream https://github.com/podium/uinta\n   ```\n\n2. If you cloned a while ago, get the latest changes from upstream, and update your fork:\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git push\n   ```\n\n3. Create a new topic branch (off of `master`) to contain your feature, change,\n   or fix.\n\n   **IMPORTANT**: Making changes in `master` is discouraged. You should always\n   keep your local `master` in sync with upstream `master` and make your\n   changes in topic branches.\n\n   ```bash\n   git checkout -b  \n   ```\n\n4. Commit your changes in logical chunks. Keep your commit messages organized,\n   with a short description in the first line and more detailed information on\n   the following lines. Feel free to use Git's\n   [interactive rebase](https://help.github.com/articles/about-git-rebase/)\n   feature to tidy up your commits before making them public.\n\n5. Make sure all the tests are still passing.\n\n   ```bash\n   mix test\n   ```\n\n6. Push your topic branch up to your fork:\n\n   ```bash\n   git push origin  \n   ```\n\n7. [Open a Pull Request](https://help.github.com/articles/about-pull-requests/)\n    with a clear title and description.\n\n8. If you haven't updated your pull request for a while, you should consider\n   rebasing on master and resolving any conflicts.\n\n   **IMPORTANT**: _Never ever_ merge upstream `master` into your branches. You\n   should always `git rebase` on `master` to bring your changes up to date when\n   necessary.\n\n   ```bash\n   git checkout master\n   git pull upstream master\n   git checkout  \n   git rebase master\n   ```\n\nThank you for your contributions!","ref":"contributing.html#pull-requests"},{"type":"extras","title":"Guides - Contributing","doc":"These Guides aim to be inclusive. We use \"we\" and \"our\" instead of \"you\" and\n\"your\" to foster this sense of inclusion.\n\nIdeally there is something for everybody in each guide, from beginner to expert.\nThis is hard, maybe impossible. When we need to compromise, we do so on behalf\nof beginning users because expert users have more tools at their disposal to\nhelp themselves.\n\nThe general pattern we use for presenting information is to first introduce a\nsmall, discrete topic, then write a small amount of code to demonstrate the\nconcept, then verify that the code worked.\n\nIn this way, we build from small, easily digestible concepts into more complex\nones. The shorter this cycle is, as long as the information is still clear and\ncomplete, the better.\n\nFor formatting the guides:\n\n- We use the `elixir` code fence for all module code.\n- We use the `iex` for IEx sessions.\n- We use the `console` code fence for shell commands.\n- We use the `html` code fence for html templates, even if there is elixir code\n  in the template.\n- We use backticks for filenames and directory paths.\n- We use backticks for module names, function names, and variable names.\n- Documentation line length should hard wrapped at around 100 characters if possible.","ref":"contributing.html#guides"},{"type":"extras","title":"Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official e-mail address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement at\nengineering@therealreal.com. All complaints will be reviewed and investigated\npromptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n[https://www.contributor-covenant.org/version/2/1/code_of_conduct.html][v2.1].\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n[https://www.contributor-covenant.org/faq][FAQ]. Translations are available at\n[https://www.contributor-covenant.org/translations][translations].\n\n[homepage]: https://www.contributor-covenant.org\n[v2.1]: https://www.contributor-covenant.org/version/2/1/code_of_conduct.html\n[Mozilla CoC]: https://github.com/mozilla/diversity\n[FAQ]: https://www.contributor-covenant.org/faq\n[translations]: https://www.contributor-covenant.org/translations","ref":"code_of_conduct.html#attribution"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}